module Sudoku where

data Sudoku = Sudoku Int [[Maybe Int]]

instance (Show Sudoku) where
    show (Sudoku _ ls) = "\n"++(concat . intersperse "\n" $ map row ls)++"\n" where
        row = concat . map showEntry
        showEntry Nothing = "."
        showEntry (Just x) = show x

-- (entry i j s) is the entry of s at position (i,j), zero-indexed
entry :: Int -> Int -> Sudoku -> Maybe Int
-- You write this one.

-- (replace i j x s) is the Sudoku board you get by replacing the entry at (i,j) with x
replace :: Int -> Int -> Maybe Int -> Sudoku -> Sudoku
replace i j x (Sudoku n ls) = Sudoku n (replace' i j x ls) where
    replace' 0 j x (l:ls) = (a ++ (x:b)):ls where (a, (_:b)) = splitAt j l
    replace' i j x (l:ls) = l:replace' (i-1) j x ls
    
noConflict :: Maybe Int -> Maybe Int -> Bool
noConflict (Just x) (Just y) = x /= y
noConflict _ _ = True

-- (goodEntry i j s) tests whether the entry at (i,j) satisfies the rules of Sudoku
goodEntry :: Int -> Int -> Sudoku -> Bool
goodEntry i j s@(Sudoku n _) = goodRow && goodCol && goodBlock where
    r = n^2
    e = entry i j s
    block k l = (k `div` n, l `div` n)
    goodRow = all (noConflict e) [entry i k s | k <- [0..r-1], k /= j]
    goodCol = all (noConflict e) [entry k j s | k <- [0..r-1], k /= i]
    goodBlock = all (noConflict e) [entry k l s | k <- [0..r-1], l <- [0..r-1], block k l == block i j, (k,l) /= (i,j)]
    
-- (firstSpot s) is the first position (in some order) at which there is a Nothing in s.
firstSpot :: Sudoku -> Maybe (Int, Int)
-- You write this one.

sample = Sudoku 2 [[Just 1, Nothing, Nothing, Nothing],
                   [Nothing, Nothing, Just 3, Nothing],
                   [Nothing, Just 1, Nothing, Nothing],
                   [Nothing, Nothing, Nothing, Just 4]]

sample' = Sudoku 2 [[Just 1, Nothing, Nothing, Nothing],
                    [Nothing, Nothing, Just 3, Nothing],
                    [Nothing, Nothing, Nothing, Nothing],
                    [Nothing, Nothing, Nothing, Just 4]]

sample'' = Sudoku 2 [[Just 1, Nothing, Nothing, Nothing],
                     [Nothing, Nothing, Just 3, Nothing],
                     [Nothing, Just 3, Nothing, Nothing],
                     [Nothing, Nothing, Nothing, Just 4]]

-- (solutions s) is the list of all boards you can get as a solution to s
solutions :: Sudoku -> [Sudoku]
-- You write this one.
